# Setting the environment

Set values in `config.json` based on your specific database
Currently the application supports `mssql`, `mysql`, and `oracledb` databases by setting the `databaseType` value

Default configuration is below in case something is accidentally changed or deleted

```
{
  "databaseType": "mysql",
  "oracleConfig": {
    "user": "user",
    "password": "password",
    "connectionString": "oracle-db-url:1521/ORCL",
    "libraryPath": "./oracle-client"
  },
  "mysqlConfig": {
    "host": "mysql-db-url",
    "user": "user",
    "password": "password",
    "databaseName": "my_database",
    "port": 3306
  },
  "mssqlConfig": {
    "host": "mssql-db-url",
    "user": "user",
    "password": "password",
    "databaseName": "my_database",
    "port": 1433,
    "instanceName": "database-instance"
  },
  "postgresConfig": {
    "user": "user",
    "password": "password",
    "host": "postgres-db-url",
    "database": "my_database".
    "port": 5432,
  },
  "logLevel": "info",
  "capacityHost": "host_provided_by_capacity",
  "capacitySecret": "secret_provided_by_capacity",
  "notificationEmail": "email_address_for_sending_notification"
}
```

`capacityHost` and `capacitySecret` are provided by Capacity and should not be changed, these values are required for the server to connect to Capacity's servers.

## Oracle Configuration

Capacity LiveDB uses [node-oracledb](http://oracle.github.io/node-oracledb/) for oracledb connections
This library requires you to have an Oracle Instant Client installed on the server where this is running and configured via setting `oracleConfig.libraryPath` in `settings.json`

The following properties are required in `oracleConfig` when `databaseType` is set to `oracledb`:

- `user` -- the database user
- `password` -- the database password
- `connectionString` -- the host:port/instance_name string that conforms to `[//]host_name[:port][/service_name][:server_type][/instance_name]`
- `libraryPath` -- the path to the oracle instant client, either relative to `server.js` or absolute.

Oracle seems to sometimes require a valid hostname to be set on the machine hosting the client for loopback address 127.0.1.1 (as opposed to the usual 127.0.0.1). If you're getting `"ORA-24454: client host name is not set"` errors, the following command may help resolve them:

`` echo -e "127.0.1.1\t`hostname`" | sudo tee -a /etc/hosts ``

More on this error can be found at [this stackoverflow thread](https://dba.stackexchange.com/questions/167477/ora-24454-client-host-name-is-not-set-when-connecting-from-ubuntu-instance-on?atw=1)

## MySQL Configuration

Capacity LiveDB uses [node-mysql2](https://www.npmjs.com/package/mysql2) for mysql connections

The following properties are required in `mysqlConfig` when `databaseType` is set to `mysql`

- `user` - the database user
- `password` - the database password
- `databaseName` - the name of the database LiveDB should connnect to
- `host` -- the url to the mysql database
- `port` - the port the mysql database is running on

### MySQL SSL Connections

- Three methods of configuring an SSL connections are supported via the optional `mysqlConfig.ssl` value in `config.json` when `databaseType` is set to `mysql`

1. Amazon RDS -- `"ssl": "Amazon RDS"`
2. SSL with a CA -- `"ssl": { "ca": "./name-of-ca-file.pem" }`
3. SSL with Key and Cert -- `"ssl": { "key": "./name-of-key-file.pem", "cert": "./name-of-cert-file.pem" }`

If using option 1, the library automatically fetches `rds-combined-ca-bundle.pem` and uses it. If using option 2 or 3 the file paths are relative to `server.js`

For example, if using Amazon RDS but specifying the CA file, you would place the file `rds-combined-ca-bundle.pem` in the same directory as `server.js` and set `"ssl": { "ca": "./rds-combined-ca-bundle.pem" }`

## MSSql Configuration

Capacity LiveDB uses [node-mssql](https://www.npmjs.com/package/mssql) with the tedious driver for mssql connections

The following properties are required in `mssqlConfig` when `databaseType` is set to `mssql`

- `user` - the database user
- `password` - the database password
- `databaseName` - the name of the database LiveDB should connect to
- `host` -- the url to the mssql database
- `port` - the port the mssql database is running on

The following properties are optional in `mssqlConfig` when `databaseType` is set to `mssql`

- `instanceName` -- the named instance LiveDB should connec to, can optonally be defined in `host` by setting `host-url\\instanceName`

## PostgreSQL Configuration

Capacity LiveDB uses [node-postgres](https://www.npmjs.com/package/pg) for postgres connections

The following properties are required in `postgresConfig` when `databaseType` is set to `postgres`

- `user` - the database user
- `password` - the database password
- `database` - the name of the database LiveDB should connnect to
- `host` -- the url to the postgres database
- `port` - the port the postgres database is running on

## config.json Type Definition

The `config.json` file must conform to the following typescript definition:

```
enum DatabaseType {
  mysql = 'mysql',
  mssql = 'mssql',
  oracledb = 'oracledb',
  postgres = 'postgres',
}

enum LogLevel {
  info = 'info',
  warning = 'warning',
  error = 'error',
}

type MSSqlConfig = {
  host: string
  user: string
  password: string
  databaseName: string
  port: string
  instanceName?: string
}

type MySqlSSLConfig = string | { ca: string } | { key: string; cert: string }

type MySqlConfig = {
  host: string
  user: string
  password: string
  databaseName: string
  port: string
  ssl?: MySqlSSLConfig
}

type OracleConfig = {
  user: string
  password: string
  connectionString: string
  libraryPath: string
}

type PostgresConfig = {
  host: string
  user: string
  password: string
  database: string
  port: string
}

type ConfigFile = {
  databaseType: DatabaseType
  oracleConfig?: OracleConfig
  mysqlConfig?: MySqlConfig
  mssqlConfig?: MSSqlConfig
  postgresConfig?: PostgresConfig
  capacityHost: string
  capacitySecret: string
  notificationEmail?: string
  logLevel?: LogLevel
}
```

# Starting the server

The command `node server.js` will automatically start the server listening at port 4201 and start displaying logs, the port can be overriden by passing a `PORT` value with the command when running the server -- for example `PORT=4400 node server.js`

The application assumes it is being started from the directory in which `server.js` resides, so do not issue the start command from a different folder.

Upon server startup it will attempt to connect with Capacity's server and will log a message upon a successful connection or if there is a connection error

There is a configuration UI available at `/configuration` that can be accessed via a web browser to assist in creating queries, but manual configuration is also possible. If using the default port, the URL would be `http://localhost:4201/configuration`

# Logging

The server logs each incoming request over the persistent connection as well as the raw data contained in each request and the raw data being returned to Capacity at the `info` log level.
The server logs all encountered errors on the `error` log level.
No other log levels are utilized.

Log level can be configured by setting the `logLevel` value in `config.json`. Accepted values are `info` and `error`, default is `info`.
For example, to log everything, set `logLevel` to `info`, but to log only errors, set `logLevel` to `error`

# Queries

Queries are stored as flat `json` files in the included `queries` directory with the following schema:

```json
{
  "inputs": [{ "name": "string", "type": "string" }],
  "outputs": [{ "name": "string", "type": "string" }]
}
```

Query strings must be defined using `@variable_name` syntax, and any included variables in a query string must be defined in the `inputs` array

For example, if I want to query my `users` table by name, and return the `id` and `name` of the result, my query might look something like:

`get_user_by_name.json`

```json
{
  "inputs": [{ "name": "name", "type": "VarChar" }],
  "outputs": [
    { "name": "id", "type": "Int" },
    { "name": "name", "type": "VarChar" }
  ],
  "query": "select * from users where name = @name"
}
```

Each individual database has a its own list of supported types for inputs and outputs

## MSSQL Types

- Bit
- BigInt
- Decimal
- Float
- Int
- Money
- Numeric
- SmallInt
- SmallMoney
- Real
- TinyInt
- Char
- NChar
- Text
- NText
- VarChar
- NVarChar
- Xml
- Time
- Date
- DateTime
- DateTime2
- DateTimeOffset
- SmallDateTime
- UniqueIdentifier
- Variant
- Binary
- VarBinary
- Image
- UDT
- Geography
- Geometry

## MySQL Types

- INT
- TINYINT
- SMALLINT
- MEDIUMINT
- BIGINT
- FLOAT
- DOUBLE
- DECIMAL
- DATE
- DATETIME
- TIMESTAMP
- TIME
- YEAR
- CHAR
- VARCHAR
- BLOB
- TEXT
- TINYBLOB
- TINYTEXT
- MEDIUMBLOB
- MEDIUMTEXT
- LONGBLOB
- LONGTEXT
- ENUM

## OracleDB Types

- BFILE
- BINARY_DOUBLE
- BINARY_FLOAT
- BINARY_INTEGER
- BLOB
- BOOLEAN
- CHAR
- CLOB
- CURSOR
- DATE
- INTERVAL_DS
- INTERVAL_YM
- LONG
- LONG_RAW
- NCHAR
- NCLOB
- NUMBER
- NVARCHAR
- OBJECT
- RAW
- ROWID
- TIMESTAMP
- TIMESTAMP_LTZ
- TIMESTAMP_TZ
- VARCHAR

## PostgreSQL Types

- ABSTIME
- ACLITEM
- BIT
- BOOL
- BPCHAR
- BYTEA
- CHAR
- CID
- CIDR
- CIRCLE
- DATE
- FLOAT4
- FLOAT8
- GTSVECTOR
- INET
- INT2
- INT4
- INT8
- INTERVAL
- JSON
- JSONB
- MACADDR8
- MACADDR
- MONEY
- NUMERIC
- OID
- PATH
- PG_DEPENDENCIES
- PG_LSN
- PG_NDISTINCT
- PG_NODE_TREE
- POLYGON
- REFCURSOR
- REGCLASS
- REGCONFIG
- REGDICTIONARY
- REGNAMESPACE
- REGOPER
- REGOPERATOR
- REGPROC
- REGPROCEDURE
- REGROLE
- REGTYPE
- RELTIME
- SMGR
- TEXT
- TID
- TIME
- TIMESTAMP
- TIMESTAMPTZ
- TIMETZ
- TINTERVAL
- TSQUERY
- TSVECTOR
- TXID_SNAPSHOT
- UUID
- VARBIT
- VARCHAR
- XID
- XML

Capacity does not have access via the persistent connection to the values stored in the `query` field or any values placed in the `config.json` file besides those we provide.

Capacity can get a list of all file names, which are used to send commands, as well as each file's `inputs` and `outputs` array.
